/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


/**
 * @file
 *
 * @author Joseph Schlecht
 *
 * @cclicense
 *
 * @brief Provides a mapping between haplo groups and their sequential ordering.
 */


#ifndef HAPLO_GROUPS_H
#define HAPLO_GROUPS_H


#include <config.h>

#include <stdlib.h>
#include <inttypes.h>

#include <jwsc/base/error.h>
#include <jwsc/vector/vector.h>


/** @brief Type of ancestry relating two haplo groups. */
typedef enum 
{
    HAPLO_ANCESTOR_NONE=0,
    HAPLO_ANCESTOR_DIRECT,
    HAPLO_ANCESTOR_INDIRECT
}
Haplo_ancestor_type;


/** @brief Reads the haplo groups from a file. */
Error* read_haplo_groups(const char* fname);


/** @brief Looks up the index for a haplo group based on its label. */
Error* lookup_haplo_group_index_from_label
(
    uint32_t*   index_out, 
    const char* label
);


/** @brief Looks up a haplo group label from its index. */
Error* lookup_haplo_group_label_from_index
(
    const char** label_out, 
    uint32_t     index
);


/** @brief Looks up the index of the immediate ancestor for the haplogroup specified by the index **/

Error* lookup_haplo_group_ancestor_index_from_haplo_group_index
(
    uint32_t*   index_out, 
    uint32_t     index
);

/** @brief Returns the number of haplo groups represented. */
uint32_t get_num_haplo_groups();


/** @brief Returns whether a haplo group label is an ancestor of another. */
uint8_t is_ancestor(uint32_t index_1, uint32_t index_2);


/** @brief Finds the lowest common ancestor between two haplo group labels. */
Haplo_ancestor_type find_ancestor_index_of_pair
(
    uint32_t* index_out, 
    uint32_t  index_1, 
    uint32_t  index_2
);


/** 
 * @brief Finds the lowest common ancestor between a set of haplo group labels.
 */
Haplo_ancestor_type find_ancestor_index_of_set
(
    uint32_t*         index_out, 
    const Vector_u32* indices
);

#endif
