/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


/**
 * @file
 *
 * @author Joseph Schlecht
 *
 * @cclicense
 *
 * @brief Output related functions.
 */


#ifndef OUTPUT_H
#define OUTPUT_H


#include <config.h>

#include <stdlib.h>
#include <stdio.h>
#include <inttypes.h>
#include <assert.h>

#include <libxml/tree.h>

#include <jwsc/base/error.h>
#include <jwsc/vector/vector.h>
#include <jwsc/matblock/matblock.h>


/** @brief Types of output file format to generate. */
typedef enum
{
    HAPLO_OUTPUT_TXT,
    HAPLO_OUTPUT_CSV,
    HAPLO_OUTPUT_XML
}
Haplo_output_format;


/** @brief Types of field separation in an output file. */
typedef enum
{
    HAPLO_SEP_PREFIX,
    HAPLO_SEP_SUFFIX,
    HAPLO_SEP_BOTH,
    HAPLO_SEP_NONE
}
Haplo_field_sep;


/** @brief Open an output file for writing. */
Error* open_output
(
    FILE**      fp_out, 
    xmlDoc**    xml_doc_out, 
    const char* xml_name,
    const char* xml_dtd,
    const char* fname
);


/** @brief Close a file used for output writing. */
Error* close_output(FILE* fp, xmlDoc* doc, const char* fname);


/** @brief Writes a sample ids to a file or an xml node. */
void write_ids
(
    const Matblock_u8* ids,
    uint32_t           i,
    Haplo_field_sep    field_sep,
    FILE*              fp,
    xmlNode*           xml_node
);


/** @brief Writes a sample label to a file or an xml node. */
void write_label
(
    const Vector_u32* labels,
    uint32_t          i,
    Haplo_field_sep   field_sep,
    FILE*             fp,
    xmlNode*          xml_node
);


/** @brief Writes a label confidence value to a file stream. */
void write_conf
(
    const Vector_d* confs,
    uint32_t        i,
    Haplo_field_sep field_sep,
    FILE*           fp,
    xmlNode*        xml_node
);


/** @brief Writes a prediction. */
void write_prediction
(
    const char*       type,
    const Vector_u32* labels,
    const Vector_d*   confs,
    uint32_t          i,
    Haplo_field_sep   field_sep,
    FILE*             fp,
    xmlNode*          xml_node
);


/** @brief Writes the ancestor to a file stream. */
void write_ancestor_label
(
    const Vector_u32*  ancestor_types,
    const Vector_u32*  ancestor_labels,
    uint32_t           i,
    Haplo_field_sep    field_sep,
    FILE*              fp,
    xmlNode*           xml_node
);

#endif
